PredicateEvaluator

Trait PredicateEvaluator 

pub trait PredicateEvaluator: AsAny {
    // Required method
    fn evaluate(
        &self,
        batch: &dyn EngineData,
    ) -> DeltaResult<Box<dyn EngineData>>;
}
Trait for implementing a Predicate evaluator.

It contains one Predicate which can be evaluated on multiple ColumnarBatches. Connectors can implement this trait to optimize the evaluation using the connector specific capabilities.

Required Methods

fn evaluate(&self, batch: &dyn EngineData) -> DeltaResult<Box<dyn EngineData>>

Evaluate the predicate on a given EngineData.

Produces one boolean value for each row of the input.

Implementors