pub struct RetryableTransaction<S = ExistingTable> {
pub transaction: Transaction<S>,
pub error: Error,
}Expand description
A transaction that failed to commit due to a retryable error (e.g. IO error). The transaction
can be recovered with RetryableTransaction::transaction and retried without rebasing. The
associated error can be inspected via RetryableTransaction::error.
Fields
transaction: Transaction<S>The transaction that failed to commit due to a retryable error.
error: ErrorTransient error that caused the commit to fail.