TryFromArrow

Trait TryFromArrow 

pub trait TryFromArrow<ArrowType>: Sized {
    // Required method
    fn try_from_arrow(t: ArrowType) -> Result<Self, ArrowError>;
}
Available on (crate features default-engine-native-tls or default-engine-rustls or arrow-conversion) and crate feature arrow-conversion only.
Convert an arrow type into a kernel type (a similar TryIntoKernel trait is automatically implemented for all types that implement TryFromArrow)

Required Methods

fn try_from_arrow(t: ArrowType) -> Result<Self, ArrowError>

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors