Module log_segment

Module log_segment 

Available on crate feature internal-api only.
Represents a segment of a delta log. LogSegment wraps a set of checkpoint and commit files.

Structs

ActionsWithCheckpointInfo
Result of reading actions from a log segment, containing both the actions iterator and checkpoint metadata.
CheckpointReadInfo
Information about checkpoint reading for data skipping optimization.
LogSegment
A LogSegment represents a contiguous section of the log and is made of checkpoint files and commit files and guarantees the following: 1. Commit file versions will not have any gaps between them. 2. If checkpoint(s) is/are present in the range, only commits with versions greater than the most recent checkpoint version are retained. There will not be a gap between the checkpoint version and the first commit version. 3. All checkpoint_parts must belong to the same checkpoint version, and must form a complete version. Multi-part checkpoints must have all their parts.
LogSegmentFiles
Represents the set of log files found during a listing operation in the Delta log directory.