Engine

Trait Engine 

pub trait Engine: AsAny {
    // Required methods
    fn evaluation_handler(&self) -> Arc<dyn EvaluationHandler> ;
    fn storage_handler(&self) -> Arc<dyn StorageHandler> ;
    fn json_handler(&self) -> Arc<dyn JsonHandler> ;
    fn parquet_handler(&self) -> Arc<dyn ParquetHandler> ;
}
The Engine trait encapsulates all the functionality an engine or connector needs to provide to the Delta Kernel in order to read the Delta table.

Engines/Connectors are expected to pass an implementation of this trait when reading a Delta table.

Required Methods

fn evaluation_handler(&self) -> Arc<dyn EvaluationHandler>

Get the connector provided EvaluationHandler.

fn storage_handler(&self) -> Arc<dyn StorageHandler>

Get the connector provided StorageHandler

fn json_handler(&self) -> Arc<dyn JsonHandler>

Get the connector provided JsonHandler.

fn parquet_handler(&self) -> Arc<dyn ParquetHandler>

Get the connector provided ParquetHandler.

Implementors

Source

impl<E: TaskExecutor> Engine for DefaultEngine<E>

Available on (crate features default-engine-native-tls or default-engine-rustls or arrow-conversion) and crate feature default-engine-base only.