Module transforms

Module transforms 

Shared transform infrastructure.

Macros

transform_output_type
Defines a transform’s Output and Residual associated types.

Structs

ExpressionDepthChecker
An expression “transform” that doesn’t actually change the expression at all. Instead, it measures the maximum depth of a expression, with a depth limit to prevent stack overflow. Useful for verifying that a expression has reasonable depth before attempting to work with it.
SchemaDepthChecker
A schema “transform” that doesn’t actually change the schema at all. Instead, it measures the maximum depth of a schema, with a depth limit to prevent stack overflow. Useful for verifying that a schema has reasonable depth before attempting to work with it.

Traits

Carrier
Carrier abstraction for transform outputs.
ExpressionTransform
Generic framework for recursive bottom-up transforms of expressions and predicates.
SchemaTransform
Generic framework for describing recursive bottom-up schema transforms.